﻿using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using StatisticsService.Domain.Queries;
using StatisticsService.Dto;
using ZData.Iam.Enforcer.Client;

namespace StatisticsService.Api.Controllers
{
    [Route("api/v1/resources")]
    [ApiController]
    [Authorize]
    public class ResourcesController : ControllerBase
    {
        private readonly IEnforcer enforcer;
        private readonly IMediator mediator;


        public ResourcesController(IEnforcer enforcer, IMediator mediator)
        {
            this.enforcer = enforcer;
            this.mediator = mediator;
        }

        /// <summary>
        /// Gets all resources of a given type
        /// </summary>
        /// <param name="resourceType"> Defines what resource type to request</param>
        /// <remarks>Returns a list of all resources that matches the input resource type.</remarks>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<ResourcesDto>>> GetAllResources([FromQuery, BindRequired] string resourceType)
        {
            var resources = await mediator.Send(new GetAllResourcesQuery(resourceType));

            if (resources == null)
                return NotFound();

            return Ok(resources);
        }

        /// <summary>
        /// Gets all available resource types
        /// </summary>
        /// <remarks>Returns a list of all valid resource types</remarks>
        [AllowAnonymous]
        [HttpGet("types")]
        public async Task<ActionResult<IEnumerable<string>>> GetAllResourceTypes()
        {
            var resourceTypes = await mediator.Send(new GetAllResourceTypesQuery());

            if (resourceTypes == null)
                return NotFound();

            return Ok(resourceTypes);
        }
    }
}