﻿using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using StatisticsService.Domain;
using StatisticsService.Domain.Filters;
using StatisticsService.Domain.Queries;
using StatisticsService.Dto;
using ZData.Iam.Enforcer.Client;

namespace StatisticsService.Api.Controllers
{
    /// <summary>
    /// Controller for Statistics endpoint
    /// </summary>
    [Route("api/v1/statistics")]
    [ApiController]
    [Authorize]
    public class StatisticsController : ControllerBase
    {
        private readonly IEnforcer enforcer;
        private readonly IMediator mediator;

        /// <summary>
        /// Constructor for Statistics controller
        /// </summary>
        /// <param name="enforcer"></param>
        /// <param name="mediator"></param>
        public StatisticsController(IEnforcer enforcer, IMediator mediator)
        {
            this.enforcer = enforcer;
            this.mediator = mediator;
        }

        /// <summary>
        /// Gets all statistics for a given event and time type
        /// </summary>
        /// <remarks>Returns a list of statistics for a requested event type for a requested date range</remarks>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<StatisticsDto>>> GetStatistics([FromQuery] StatisticsFilter filter)
        {
            var statistics = await mediator.Send(new GetStatisticsQuery(filter));

            if (statistics == null)
                return NotFound();

            return Ok(statistics);
        }

        /// <summary>
        /// Gets statistics associated with a resource ID
        /// </summary>
        /// <remarks>Returns a list of statistics associated with a resource for a requested event type within requested date range</remarks>
        [HttpGet("{ResourceId}")]
        public async Task<ActionResult<IEnumerable<StatisticsDto>>> GetStatisticsByResource([FromQuery] StatisticsFilterRID filter)
        {
            var statistics = await mediator.Send(new GetStatisticsByResourceQuery(filter));

            if(statistics == null)
                return NotFound();

            return Ok(statistics);
        }

        /// <summary>
        /// Gets all aggregated statistics for a given event and time type
        /// </summary>
        /// <remarks>Returns a list of dates with counts of requested statistics within requested date range.</remarks>
        [HttpGet("aggregate")]
        public async Task<ActionResult<IEnumerable<AggregatedStatisticsDto>>> GetAggregatedStatistics([FromQuery] StatisticsFilter filter)
        {
            var statistics = await mediator.Send(new GetAggregatedStatisticsQuery(filter));

            if (statistics == null)
                return NotFound();

            return Ok(statistics);
        }

        /// <summary>
        /// Gets aggregated statistics associated with a resource ID
        /// </summary>
        /// <remarks>Returns a list of dates with counts of requested statistics for a resource within requested date range.</remarks>
        [HttpGet("aggregate/{ResourceId}")]
        public async Task<ActionResult<IEnumerable<AggregatedStatisticsDto>>> GetAggregatedStatisticsByResource([FromQuery] StatisticsFilterRID filter)
        {
            var statistics = await mediator.Send(new GetAggregatedStatisticsByResourceQuery(filter));

            if (statistics == null)
                return NotFound();

            return Ok(statistics);
        }

        /// <summary>
        /// Gets all available event types
        /// </summary>
        /// <remarks>Returns a list of all valid event types</remarks>
        [AllowAnonymous]
        [HttpGet("events/types")]
        public async Task<ActionResult<IEnumerable<string>>> GetAllEventTypes()
        {
            var eventTypes = await mediator.Send(new GetAllEventTypesQuery());

            if (eventTypes == null)
                return NotFound();

            return Ok(eventTypes);
        }
    }
}
