﻿using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using StatisticsService.Domain.Filters;
using StatisticsService.Domain.Queries;
using StatisticsService.Dto;
using ZData.Iam.Enforcer.Client;

namespace StatisticsService.Api.Controllers
{
    [Route("api/v1/users")]
    [ApiController]
    [Authorize]
    public class UsersController : ControllerBase
    {
        private readonly IEnforcer enforcer;
        private readonly IMediator mediator;

        public UsersController(IEnforcer enforcer, IMediator mediator)
        {
            this.enforcer = enforcer;
            this.mediator = mediator;
        }

        /// <summary>
        /// Get all users
        /// </summary>
        /// <remarks>Returns a list containing all users registered within requested date range</remarks>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<UserDto>>> GetAllUsers([FromQuery] BaseFilter filter)
        {
            var users = await mediator.Send(new GetAllUsersQuery(filter));

            if (users == null)
                return NotFound();

            return Ok(users);
        }

        /// <summary>
        /// Gets all users associated with a given resource
        /// </summary>
        /// <remarks>Returns a list containing all users associated with a resource registered within requested date range</remarks>
        [HttpGet("{ResourceId}")]
        public async Task<ActionResult<IEnumerable<UserDto>>> GetUsersByResourceId([FromQuery] BaseFilterRID filter)
        {
            var users = await mediator.Send(new GetUsersByResourceIdQuery(filter));

            if (users == null)
                return NotFound();

            return Ok(users);
        }

        /// <summary>
        /// Gets all active users
        /// </summary>
        /// <remarks>Returns a list of dates with counts of active users within requested date range.</remarks>
        [HttpGet("all/active")]
        public async Task<ActionResult<ActiveResourceDto>> GetAllActiveUsers([FromQuery] BaseFilter filter)
        {
            var users = await mediator.Send(new GetAllActiveUsersQuery(filter));

            if (users == null)
                return NotFound();

            return Ok(users);
        }

        /// <summary>
        /// Gets all active users associated with a given resource
        /// </summary>
        /// <remarks>Returns a list of dates with counts of active users associated with requested resource within requested date range.</remarks>
        [HttpGet("{ResourceId}/active")]
        public async Task<ActionResult<ActiveResourceDto>> GetActiveUsersByResourceId([FromQuery] BaseFilterRID filter)
        {
            var users = await mediator.Send(new GetActiveUsersByResourceIdQuery(filter));

            if (users == null)
                return NotFound();

            return Ok(users);
        }
    }
}