﻿using Microsoft.EntityFrameworkCore;
using Resources;
using StatisticsService.Api.Clients;
using StatisticsService.Api.Extensions.QueryExtensions;
using StatisticsService.Constants;
using StatisticsService.Database;
using StatisticsService.Database.Models;
using StatisticsService.Domain;
using StatisticsService.Dto;
using StatisticsService.Interfaces.Repository;

namespace StatisticsService.Api.Repository
{
    /// <summary>
    /// A repository for resources
    /// </summary>
    public class ResourcesRepository : BaseRepository, IResourcesRepository
    {
        private readonly ICachedResourceProviderClient resourceServiceClient = null!;
        private readonly IZarManagementClient zarManagementClient = null!;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="context"></param>
        public ResourcesRepository(StatisticsServiceDbContext context) : base(context) { }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="context"></param>
        /// <param name="resourceServiceClient">ResourceService Provider Client</param>
        /// <param name="zarManagementClient">ZData Access Resource Management Client (ZAR)</param>
        public ResourcesRepository(StatisticsServiceDbContext context, ICachedResourceProviderClient resourceServiceClient, IZarManagementClient zarManagementClient) : base(context)
        {
            this.resourceServiceClient = resourceServiceClient;
            this.zarManagementClient = zarManagementClient;
        }

        /// <summary>
        /// Returns all resources of a given type <paramref name="resourceType"/>
        /// </summary>
        /// <param name="resourceType">Type of resource</param>
        /// <returns>List of resources of a given type</returns>
        public async Task<IEnumerable<ResourcesDto>> GetAllResourcesAsync(string resourceType)
        {
            var resources = new List<ResourcesDto>();

            switch(resourceType)
            {
                case Constants.Resources.Company:
                    var resourceIds = await GetResourceIds(resourceType);

                    var iterations = 0;
                    while (iterations * 2000 < resourceIds.Count())
                    {
                        var request = new GetResourceRequestBatch();
                        request.Requests.AddRange(resourceIds.Skip(iterations * 2000).Take(2000).Select(r => new GetResourceRequest { ResourceId = $"{resourceType}-{r}" }));

                        await foreach (var response in resourceServiceClient.GetResourceBatchAsync(request))
                        {
                            resources.Add(new ResourcesDto()
                            {
                                Id = response.Id[(response.Id.IndexOf("-") + 1)..],
                                Type = response.Type,
                                Name = response.Name.Trim()
                            });
                        }

                        iterations++;
                    }

                    break;
                case Constants.Resources.Partner:
                    var zarResponse = await zarManagementClient.GetSubjects("apikey");
                    resources = zarResponse.Items.Select(item => new ResourcesDto()
                    {
                        Id = item.Id[(item.Id.IndexOf("-") + 1)..],
                        Type = Constants.Resources.Partner,
                        Name = item.Name.Trim()
                    }).ToList();

                    break;

                default:
                    throw new ArgumentOutOfRangeException("Not a valid resourceType.");
            }

            return resources.OrderBy(x => x.Name);
        }

        /// <summary>
        /// Returns all resource Ids of a given type <paramref name="resourceType"/> that has been active last month
        /// </summary>
        /// <param name="resourceType">Type of resource</param>
        /// <returns>List of resource Ids of a given type active last month</returns>
        public async Task<IEnumerable<string>> GetResourceIds(string resourceType)
        {
            var fromDate = DateTime.Now.AddMonths(-1).Date;

            return await Context.AggregatedEventsDay
                .Where(x => x.ResourceType == resourceType && x.EventTime >= fromDate)
                .Select(x => x.TopicId)
                .Distinct()
                .ToListAsync();
        }
    }
}
