﻿using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace StatisticsService.Api.Workers
{
    public abstract class CronJobService : IHostedService, IDisposable
    {
        private System.Timers.Timer? timer;
        private readonly CronExpressionCollection expression;
        private readonly TimeZoneInfo timeZoneInfo;
        private readonly ILogger<BankServiceCronService> logger;

        protected CronJobService(TimeZoneInfo timeZoneInfo, ILogger<BankServiceCronService> logger, params string[] cronExpressions)
            : this(timeZoneInfo, logger)
        {
            expression.AddExpressions(cronExpressions);
        }

        private CronJobService(TimeZoneInfo timeZoneInfo, ILogger<BankServiceCronService> logger)
        {
            this.logger = logger;
            this.expression = new CronExpressionCollection();
            this.timeZoneInfo = timeZoneInfo;
        }

        public virtual async Task StartAsync(CancellationToken cancellationToken)
        {
            // Need to call this method to prevent BackgroundService from blocking rest of the application
            await Task.Yield();

            logger.LogInformation("Start BankService Cron Job Service: {cronServiceName} with expression {cronExpression}", this.GetType().Name, expression);
            await ScheduleJob(cancellationToken);
        }

        protected virtual async Task ScheduleJob(CancellationToken cancellationToken)
        {
            var next = expression.GetNextOccurrence(DateTimeOffset.Now, timeZoneInfo);
            if (next.HasValue)
            {
                var delay = next.Value - DateTimeOffset.Now;
                if (delay.TotalMilliseconds <= 0)   // prevent non-positive values from being passed into Timer
                {
                    await ScheduleJob(cancellationToken);
                }
                logger.LogTrace("BankService Cron Job scheduled at {next} (in {milliseconds}ms)", next, delay.TotalMilliseconds);
                timer = new System.Timers.Timer(delay.TotalMilliseconds);
                timer.Elapsed += async (sender, args) =>
                {
                    timer.Dispose();  // reset and dispose timer
                    timer = null;

                    if (!cancellationToken.IsCancellationRequested)
                    {
                        await DoWork(cancellationToken);
                    }

                    if (!cancellationToken.IsCancellationRequested)
                    {
                        await ScheduleJob(cancellationToken);    // reschedule next
                    }
                };
                timer.Start();
            }
            await Task.CompletedTask;
        }

        public virtual async Task DoWork(CancellationToken cancellationToken)
        {
            await Task.Delay(5000, cancellationToken);  // do the work
        }

        public virtual async Task StopAsync(CancellationToken cancellationToken)
        {
            timer?.Stop();
            await Task.CompletedTask;
        }

        public virtual void Dispose()
        {
            timer?.Dispose();
        }
    }
}
