﻿using System.Collections.Generic;
using System.Threading.Tasks;

namespace StatisticsService.Constants
{
    /// <summary>
    /// Class of constants, used to define available events in API and translate shorthand names to full names.
    /// </summary>
    public class Events
    {
        public const string StatementsReceived = "Statements";
        public const string StatementsReceivedFull = "ZData.Events.Bank.StatementsReceived";

        public const string ReceivablesReceived = "Receivables";
        public const string ReceivablesReceivedFull = "ZData.Events.Bank.ReceivableReceived";

        public const string PaymentsSentToBank = "Payments";
        public const string PaymentsSentToBankFull = "ZData.Events.Bank.PaymentsSentToBank";


        /// <summary>
        /// Returns all valid event types as a list of strings
        /// </summary>
        public static readonly List<string> AllEventTypes = new() { StatementsReceived, ReceivablesReceived, PaymentsSentToBankFull };

        /// <summary>
        /// Asynchronously returns all valid event types as a list of strings
        /// </summary>
        public static readonly Task<List<string>> AllEventTypesAsync = Task.Run(() => new List<string> { StatementsReceived, ReceivablesReceived, PaymentsSentToBank }) ;

        /// <summary>
        /// Converts a shorthand event name to full form.
        /// </summary>
        /// <param name="e">A shorthand event name string</param>
        /// <remarks>
        /// Uses ternary instead of switch due to usage of the ToLower method.
        /// A switch method requires compile time constants, which disallows method calls.
        /// </remarks>
        /// <returns>A properly formatted event string</returns>
        public static string ConvertToFullEventName(string e)
        {
            e = e.ToLower();
            return e == StatementsReceived.ToLower() ? StatementsReceivedFull :
                   e == ReceivablesReceived.ToLower() ? ReceivablesReceivedFull :
                   e == PaymentsSentToBank.ToLower() ? PaymentsSentToBankFull :
                   "";
        }
    }
}
