﻿using FluentAssertions;
using Microsoft.EntityFrameworkCore;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using StatisticsService.Api.Repository;
using StatisticsService.Database;
using StatisticsService.Database.Models;
using StatisticsService.Domain.Filters;
using StatisticsService.Dto;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace StatisticsService.Tests
{
    [TestClass]
    public class UsersRepositoryTests
    {

        private static readonly DateTime now = DateTime.Now.Date;
        private static readonly string duplicateGUID = Guid.NewGuid().ToString();

        [TestMethod]
        public async Task GetAllUsers_ReturnsAllUsers()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<StatisticsServiceDbContext>()
                .UseInMemoryDatabase(databaseName: "test-get-all-users")
                .Options;
            var context = new StatisticsServiceDbContext(options);

            // await context.Users.AddRangeAsync();
            // await context.SaveChangesAsync();

            var usersRepository = new UsersRepository(context);
            var filter = new BaseFilter()
            {
                // Ensure dates correspond with mock data dates
                FromDate = DateTime.Parse("2022-05-03T12:35:05.5366733Z").Date,
                ToDate = DateTime.Parse("2022-05-09T08:35:05.5366733Z").Date,
            };

            // Act
            var actual = await usersRepository.GetAllUsers(filter);

            // Assert
            actual.Should().NotBeNull().And
                .HaveCount(9).And
                .AllBeOfType<UserDto>();

            // Cleanup
            context.Database.EnsureDeleted();
        }

        [TestMethod]
        public async Task GetAllUsers_Skip_ReturnsAllUsersNotSkipped()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<StatisticsServiceDbContext>()
                .UseInMemoryDatabase(databaseName: "test-get-all-users")
                .Options;
            var context = new StatisticsServiceDbContext(options);

            var usersRepository = new UsersRepository(context);
            var filter = new BaseFilter()
            {
                // Ensure dates correspond with mock data dates
                FromDate = DateTime.Parse("2022-05-03T12:35:05.5366733Z").Date,
                ToDate = DateTime.Parse("2022-05-09T08:35:05.5366733Z").Date,
                Skip = 5,
            };

            // Act
            var actual = await usersRepository.GetAllUsers(filter);

            // Assert
            actual.Should().NotBeNull().And
                .HaveCount(6).And
                .AllBeOfType<UserDto>();

            // Cleanup
            context.Database.EnsureDeleted();
        }

        [TestMethod]
        public async Task GetAllUsers_Take_ReturnsAllUsersTaken()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<StatisticsServiceDbContext>()
                .UseInMemoryDatabase(databaseName: "test-get-all-users")
                .Options;
            var context = new StatisticsServiceDbContext(options);

            var usersRepository = new UsersRepository(context);
            var filter = new BaseFilter()
            {
                // Ensure dates correspond with mock data dates
                FromDate = DateTime.Parse("2022-05-03T12:35:05.5366733Z").Date,
                ToDate = DateTime.Parse("2022-05-09T08:35:05.5366733Z").Date,
                Take = 3,
            };

            // Act
            var actual = await usersRepository.GetAllUsers(filter);

            // Assert
            actual.Should().NotBeNull().And
                .HaveCount(3).And
                .AllBeOfType<UserDto>();

            // Cleanup
            context.Database.EnsureDeleted();
        }

        [TestMethod]
        public async Task GetAllActiveUsers_ReturnsAllActiveUsers()
        {
            // Arrange
            var options = new DbContextOptionsBuilder<StatisticsServiceDbContext>()
                .UseInMemoryDatabase(databaseName: "test-get-all-users")
                .Options;
            var context = new StatisticsServiceDbContext(options);

            var usersRepository = new UsersRepository(context);
            var filter = new BaseFilter()
            {
                // Ensure dates correspond with mock data dates
                FromDate = DateTime.Parse("2022-05-03T12:35:05.5366733Z").Date,
                ToDate = DateTime.Parse("2022-05-09T08:35:05.5366733Z").Date,
            };

            // Act
            var actual = await usersRepository.GetAllActiveUsers(filter);

            // Assert
            actual.Should().NotBeNull().And
                .BeOfType<ActiveResourceDto>();

            // Cleanup
            context.Database.EnsureDeleted();
        }
    }
}
