import { ChangeEvent, useEffect, useState } from 'react'
import {
  Chart as ChartJS,
  CategoryScale,
  LinearScale,
  BarController,
  BarElement,
  LineController,
  LineElement,
  PointElement,
  Tooltip,
  ChartType,
  ChartOptions,
} from 'chart.js'
import dayjs, { ManipulateType } from 'dayjs'
import { Chart } from 'react-chartjs-2'
import { ArrowDownCircle, ArrowUpCircle } from 'react-feather'

import { CardError } from './CardError'
import { WidgetRadio } from './WidgetRadio'
import { formatNumber, percentageDifference } from 'utils'
import { useWidget } from 'utils/hooks/useWidget'
import { groupWidgetDataByDate } from 'utils/widget'
import TypeChartDataset from 'shared/interfaces/chartjs/charts.interface'
import GroupedDatasets from 'shared/interfaces/chartjs/datasets.interface'
import WidgetParameters from 'shared/interfaces/widget/widget-parameters'

ChartJS.register(
  CategoryScale,
  LinearScale,
  BarController,
  BarElement,
  LineController,
  LineElement,
  PointElement,
  Tooltip,
)

const CHART_OPTIONS: ChartOptions = {
  interaction: {
    mode: 'index',
    intersect: false,
  },
  maintainAspectRatio: false,
  responsive: true,
  plugins: {
    legend: {
      display: false,
    },
    title: {
      display: false,
    },
  },
  elements: {
    line: {
      tension: 0.5,
    },
  },
  scales: {
    x: {
      display: false,
      grid: {
        color: 'transparent',
      },
      ticks: {
        font: {
          size: 2,
        },
      },
    },
    y: {
      display: false,
      ticks: {
        display: false,
      },
    },
  },
}

const DATASETS_SETTINGS: TypeChartDataset = {
  secondary: {
    backgroundColor: 'transparent',
    borderColor: '#05c3fb',
    borderWidth: 2.5,
    categoryPercentage: 1.0,
    barPercentage: 1.0,
    barThickness: 5,
    pointBorderWidth: 3,
    pointRadius: 2,
    pointBorderColor: 'rgba(5, 195, 251, 0.5)',
    pointBackgroundColor: 'rgba(5, 195, 251, 0.6)',
  },
  green: {
    backgroundColor: 'transparent',
    borderColor: '#4ecc48',
    borderWidth: 2.5,
    categoryPercentage: 1.0,
    barPercentage: 1.0,
    barThickness: 5,
    pointBorderWidth: 3,
    pointRadius: 2,
    pointBorderColor: 'rgba(78, 204, 72, 0.5)',
    pointBackgroundColor: 'rgba(78, 204, 72, 0.6)',
  },
  pink: {
    backgroundColor: 'transparent',
    borderColor: '#f46ef4',
    borderWidth: 2.5,
    categoryPercentage: 1.0,
    barPercentage: 1.0,
    barThickness: 5,
    pointBorderWidth: 3,
    pointRadius: 2,
    pointBorderColor: 'rgba(244, 110, 244, 0.5)',
    pointBackgroundColor: 'rgba(244, 110, 244, 0.6)',
  },
  warning: {
    backgroundColor: 'transparent',
    borderColor: '#f7ba48',
    borderWidth: 2.5,
    categoryPercentage: 1.0,
    barPercentage: 1.0,
    barThickness: 5,
    pointBorderWidth: 3,
    pointRadius: 2,
    pointBorderColor: 'rgba(247, 186, 72, 0.5))',
    pointBackgroundColor: 'rgba(247, 186, 72, 0.6)',
  },
}

interface WidgetProps {
  type: 'bar' | 'line'
  variant: 'secondary' | 'pink' | 'green' | 'warning'
  title: string
  args: WidgetParameters
}

export const Widget = ({ type, variant, title, args }: WidgetProps) => {
  const [thisPeriodData, setThisPeriodData] = useState<GroupedDatasets>()
  const [periodDifference, setPeriodDifference] = useState<number>(0)
  const [periodCount, setPeriodCount] = useState<number>(0)
  const [periodType, setPeriodType] = useState<string>('week')
  const toDate = new Date()

  const fromDate = dayjs(dayjs(toDate).subtract(1, periodType as ManipulateType)).toDate()

  const thisPeriod = useWidget({
    ...args,
    to: dayjs(toDate).add(1, 'day').format('YYYY-MM-DD'),
    from: dayjs(fromDate).format('YYYY-MM-DD'),
  })

  const lastPeriod = useWidget({
    ...args,
    to: dayjs(toDate)
      .subtract(1, periodType as ManipulateType)
      .format('YYYY-MM-DD'),
    from: dayjs(fromDate)
      .subtract(2, periodType as ManipulateType)
      .format('YYYY-MM-DD'),
  })

  const isLoading = thisPeriod.isLoading || lastPeriod.isLoading
  const isError = thisPeriod.isError || lastPeriod.isError
  const data = thisPeriod.data && lastPeriod.data

  useEffect(() => {
    if (thisPeriod.data) {
      const thisPeriodGrouped = groupWidgetDataByDate(thisPeriod.data.data, fromDate, toDate)
      setThisPeriodData(thisPeriodGrouped)
      setPeriodCount(thisPeriod.data.meta.periodCount)
    }
  }, [thisPeriod.data])

  useEffect(() => {
    if (thisPeriod.data && lastPeriod.data)
      setPeriodDifference(
        percentageDifference(thisPeriod.data.meta.periodCount, lastPeriod.data.meta.periodCount),
      )
  }, [thisPeriod, lastPeriod])

  const datasets = {
    labels: thisPeriodData?.labels,
    datasets: [
      {
        data: thisPeriodData?.count,
        ...DATASETS_SETTINGS[variant],
      },
    ],
  }

  const radioChange = (e: ChangeEvent<HTMLInputElement>) => {
    setPeriodType(e.currentTarget.value)
  }

  return (
    <div className="col-lg-12 col-md-12 col-sm-12 col-xl-4">
      <div className={`card overflow-hidden ${isLoading && 'loading-skeleton'}`}>
        {!isLoading && (isError || !data) && (
          <CardError message={thisPeriod.error?.message || lastPeriod.error?.message} />
        )}
        <div className="card-body">
          <div className="d-flex">
            <div className="mt-2">
              <h6>{title}</h6>
              <h2 className="mb-0 number-font">{formatNumber(periodCount)}</h2>
            </div>
            <div className="ms-auto skeleton-img-sm">
              <Chart
                type={type as ChartType}
                options={CHART_OPTIONS}
                data={datasets}
                className="h-8 w-9 chart-dropshadow mt-2"
              />
            </div>
          </div>
          <div className="d-flex mt-1 justify-content-between">
            <span className="text-muted fs-14">
              {(!isLoading || data) && (
                <span className={`text-${variant}`}>
                  {periodDifference >= 0 ? (
                    <ArrowUpCircle className={`text-green`} size="11" />
                  ) : (
                    <ArrowDownCircle className={`text-red`} size="11" />
                  )}{' '}
                  {formatNumber(periodDifference, 0)}%
                </span>
              )}
              <span className="ms-1">since last {periodType}</span>
            </span>
            <div className="widget-radio-container">
              <WidgetRadio checkedValue={periodType} onChange={radioChange} value="week" />
              <WidgetRadio checkedValue={periodType} onChange={radioChange} value="month" />
            </div>
          </div>
        </div>
      </div>
    </div>
  )
}
