import { useEffect, useState } from 'react'
import { Helmet } from 'react-helmet-async'
import { useNavigate, useParams } from 'react-router-dom'

import { Content } from 'app/components/Content'
import { LoadingIndicator } from 'app/components/LoadingIndicator'
import { Widget } from 'app/components/widgets/Widget'
import { Graph } from 'app/components/widgets/Graph'
import { useResources } from 'utils/hooks/useResources'
import WidgetType from 'shared/enums/widget-type.enum'
import EventType from 'shared/enums/event-type.enum'
import Resource from 'shared/interfaces/resources/resource.interface'
import WidgetParameters from 'shared/interfaces/widget/widget-parameters'
import StatisticsParameters from 'shared/interfaces/graph/statistics-parameters.interface'

export const BankServicePartnerPage = () => {
  const { resourceId } = useParams()
  const navigate = useNavigate()

  const partnerQuery = useResources('partner')
  const [partner, setPartner] = useState<Resource>()

  useEffect(() => {
    if (!partnerQuery.isLoading) {
      const selectedPartner = partnerQuery.data?.find(
        (resource: Resource) => resource.id === resourceId,
      )
      if (!selectedPartner) navigate('/not-found')
      else setPartner(selectedPartner)
    }
  }, [resourceId, partnerQuery.data])

  const defaultWidgetParameters: WidgetParameters = {
    resourceId,
    resourceType: partner?.type,
    widgetType: WidgetType.clients,
  }

  const defaultGraphParameters: StatisticsParameters = {
    resourceId,
    resourceType: partner?.type,
    eventType: EventType.payments,
    timeType: 'day',
  }

  return partner ? (
    <>
      <Helmet>
        <title>{partner.name} - Bank Service</title>
      </Helmet>
      <Content
        category="Projects"
        subPage={{ label: 'Bank Service', url: '/projects/bankservice' }}
        currentPage={partner.name}
      >
        <div className="row">
          <div className="col-lg-12 col-md-12 col-sm-12 col-xl-12">
            <div className="row">
              <Widget
                type="line"
                variant="secondary"
                title="Client Activity"
                args={defaultWidgetParameters}
              />
              <Widget
                type="line"
                variant="pink"
                title="Active Accounts"
                args={{ ...defaultWidgetParameters, widgetType: WidgetType.accounts }}
              />
              <Widget
                type="line"
                variant="green"
                title="New Users"
                args={{ ...defaultWidgetParameters, widgetType: WidgetType.users }}
              />
            </div>
          </div>
        </div>
        <div className="row">
          <Graph
            type="line"
            variant="primary"
            title="Payments"
            args={{ ...defaultGraphParameters }}
          />
          <Graph
            type="line"
            variant="primary"
            title="Receivables"
            args={{ ...defaultGraphParameters, eventType: EventType.receivables }}
          />
        </div>
        <div className="row">
          <Graph
            type="line"
            variant="primary"
            title="Statements"
            args={{ ...defaultGraphParameters, eventType: EventType.statements }}
          />
        </div>
      </Content>
    </>
  ) : (
    <LoadingIndicator />
  )
}
